/*
Planned upgrades:
1> Configurable delay per-request
2> Command line addition: <hostname>
3> 
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <errno.h>

struct ConfigStruct {
	int	delay; /*Delay between requests.*/
	char *	hostname; /*Hostname to check up on.*/
	char *	conffile; /*Name of the config file.*/
};

void	intro(void);
void	Usage(void);
int	Resolve(struct sockaddr_in *, char *);
int	CheckNext(FILE *, struct sockaddr_in);
int	parse_args(int, char **, struct ConfigStruct *);

int main(int argc, char *argv[]) {
	FILE	*VulnFile;
	struct ConfigStruct	Config;
	int	Found;
	int	Open=0;
	struct sockaddr_in	sin;
	
	intro();
	if(parse_args(argc, argv, &Config)) {
		Usage();
		return(-1);
	}
	
	if( (Config.conffile == NULL) ){
		if( (VulnFile = fopen("vuln.cf", "r") ) == NULL) {
			perror("Problem opening Config File: ");
			return(-1);
		}
	} else {
		if( (VulnFile = fopen(Config.conffile, "r") ) == NULL) {
			perror("Problem opening Config File: ");
			return(-1);
		}
	}
	Resolve(&sin, Config.hostname);
	Found = 0;
	while(Found != -1) {
		Found = CheckNext(VulnFile, sin);
		if(Found == 1) {
			Open++;
		} else if(Found == -1) {
			printf("Done with scan. %d vulnerabilities found.\n", Open);
		}
		sleep(Config.delay);
	}
	return(0);
}

void Usage(void) {
	printf("\nUsage: md-webscan -h <hostname> [-d <delay>]\n");
	printf("\t-h <hostname>: Set hostname to check.\n");
	printf("\t-d <delay>: Delay for <delay> seconds between requests.\n");
	printf("\t-f <conf file>: Specify a different list of vulnerabilities to check.\n");
	printf("Defaults are:\n");
	printf("hostname:\tNo default. This is required.\n");
	printf("delay:\t\t5 seconds\n");
	printf("conf file:\tvuln.cf\n\n");
}

int Resolve(struct sockaddr_in *sin, char *host) {
	struct	hostent	*he;
	if( (he=gethostbyname(host)) == NULL ) {
		perror("Can't resolve hostname: ");
		return -1;
	}
	sin->sin_family = AF_INET;
	sin->sin_port	= htons(80);
	sin->sin_addr = *((struct in_addr *)he->h_addr);
	memset(&(sin->sin_zero), 0, 8);
	return(0);
}

int CheckNext(FILE *vf, struct sockaddr_in sin) {
	char	inBuffer[1024];
	char	outBuffer[512];
	char	fileBuffer[1024];
	char	request[256], human[256];
	char	AdvURL[256];
	char	prettybuffer[8]; /*Buffer for prettying up the output*/
	char	*pipeLocation, *resp;
	int	i=0, t=0, sk, bytes;
	int	humanLen;
	

	if(feof(vf)){ /*Done with file...*/
		return(-1);
	}
	memset(fileBuffer, 0, 1024);
	memset(inBuffer, 0, 1024);
	memset(outBuffer, 0, 512);
	memset(request, 0, 256);
	memset(human, 0, 256);
	memset(AdvURL, 0, 256);
	pipeLocation = NULL; resp = NULL;
	
	fgets(fileBuffer, 1024, vf);
	if(fileBuffer[0] == '#')/*It's a comment.*/
		return(0);
	
	pipeLocation = (char *)strstr(fileBuffer, "|");
	if(pipeLocation == NULL)/*Invalid line.*/
		return(0);
	
	while(&fileBuffer[i] != pipeLocation) {
		request[i] = fileBuffer[i];
		i++;
	}
	i++;
	pipeLocation = (char *)strstr(&fileBuffer[i], "|");
	if(pipeLocation == NULL)/*Invalid line.*/
		return(0);
	while(&fileBuffer[i] != pipeLocation) {
		human[t] = fileBuffer[i];
		i++; t++;
	}
	i++;
	t = 0;
	while((fileBuffer[i] != '\0') && (fileBuffer[i] != '\n')) {
		AdvURL[t] = fileBuffer[i];
		i++; t++;
	}
	/*Now a bit of extra carfulness..*/
	AdvURL[t++] = '\0';
	human[t++] = '\0';
	humanLen = strlen(human) + 25; /*25 chars in static strings*/
	sprintf(prettybuffer, "\x1B[%dC", (80-humanLen));
	printf("Scanning for %s:%s", human, prettybuffer);
	strcpy(outBuffer, "GET ");
	strcat(outBuffer, request);
	strcat(outBuffer, " HTTP/1.0\n\n");
	memset(inBuffer, 0, 1024);
	if( (sk = socket(AF_INET, SOCK_STREAM, 0)) == -1){
		perror("Socket creation error: ");
		return(-1);
	}
	if((connect(sk, (struct sockaddr *)&sin, sizeof(struct sockaddr))) == -1) {
		perror("Couldn't connect: ");
		return(-1);
	}
	if((send(sk, outBuffer, strlen(outBuffer), 0))==-1) {
		perror("Couldn't send: ");
		return(-1);
	}
	if((bytes=recv(sk, inBuffer, 1023, 0))==-1) {
		perror("Couldn't recv: ");
		close(sk);
		return(-1);
	}
	close(sk);
	inBuffer[bytes] = '\0';
	resp = strstr(inBuffer, "200 OK"); /* HTTP/1.1 200 Found*/
	if(resp == NULL) {
		printf("\x1B[0;32mNot Present\x1B[0;37m\n");
		return(0);
	} else {
		printf("\x1B[1;31mPresent\x1B[0;37m\a\n");
		printf("\t\tFor advisory, see: %s\n", AdvURL);
		return(1);
	}
}

void intro(void) {
/*
Want colors but don't want to use curses? See:
http://204.56.132.222/Courses/CIS415L/ANSIsys.htm
*/
	printf("\x1B[1;34mmd-webscan\x1B[0;37m: A scanner for a few well-known ");
	printf("CGI vulnerabilities.\n");
	printf("\t\tBy: \x1B[0;34m-=[\x1B[1;34mMor\x1B[1;37mdr\x1B[1;34mian\x1B[0;34m]=-\x1B[0;37m\n");
	printf("\t\tEmail: \x1B[1;32m<mordrian@hotmail.com>\x1B[0;37m\n");
	printf("\t\tVersion: \x1B[1;31m1.0.0\x1B[0;37m\n");
}

int parse_args(int argc, char **argv, struct ConfigStruct *inConf) {
	int	hostOK, delaySpec, fileSpec;
	int	i;
	hostOK = 0; delaySpec = 0; fileSpec = 0;
	i = 0;
	for(i=0; i < argc; i++) {
		if(!(strcmp(argv[i], "-h"))){
			i++;
			if( argv[i] != NULL ){
				inConf->hostname = argv[i];
				hostOK = 1;
			} else {
				return(1);
			}
		}
		if(!(strcmp(argv[i], "-d"))){
			i++;
			if( argv[i] != NULL ){
				delaySpec = 1;
				inConf->delay = atoi(argv[i]);
			}
		}
		if(!(strcmp(argv[i], "-f"))){
			i++;
			if( argv[i] != NULL ) {
				fileSpec = 1;
				inConf->conffile = argv[i];
			}
		}
	}
	if(!delaySpec)
		inConf->delay = 5;
	if(!fileSpec)
		inConf->conffile = NULL;
	if(!hostOK)
		return (1);
	return(0);
}
